/******************************************************************************/
/* PTupleSpace header file.                                                   */
/******************************************************************************/
/* This is part of the PolyKit library.                                       */
/* Copyright (C) 1998-1999 by Polycode.                                       */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may be used for free in free- and shareware */
/* programs. For commercial programs, you have to pay a fee to use it. See    */
/* the license file for more information.                                     */
/******************************************************************************/


#ifndef __PTupleSpace
#define __PTupleSpace

// PolyKit headers
#include "POS.h"
#include "PSkipList.h"
#include "PString.h"
#include "ImportExport.h"

#if __p_os == __p_beos && __POWERPC__
#pragma export on
#endif


/******************************************************************************/
/* PTupleSpace class                                                          */
/******************************************************************************/
class _IMPEXP_PKLIB PTupleSpace
{													
public:
	// Item types
	enum PType { pTypeBool, pTypeInt8, pTypeInt16, pTypeInt32, pTypeInt64, pTypeFloat, pTypeDouble, pTypePointer };

	// Constructor
	PTupleSpace(void);

	// Destructor
	virtual ~PTupleSpace(void);

	// Adding items
	void AddBool(PString name, bool aBool);
	void AddInt8(PString name, int8 aInt8);
	void AddInt16(PString name, int16 aInt16);
	void AddInt32(PString name, int32 aInt32);
	void AddInt64(PString name, int64 aInt64);
	void AddFloat(PString name, float aFloat);
	void AddDouble(PString name, double aDouble);
	void AddPointer(PString name, const void *aPointer);

	// Removing items
	void RemoveBool(PString name, int32 index = 0);
	void RemoveInt8(PString name, int32 index = 0);
	void RemoveInt16(PString name, int32 index = 0);
	void RemoveInt32(PString name, int32 index = 0);
	void RemoveInt64(PString name, int32 index = 0);
	void RemoveFloat(PString name, int32 index = 0);
	void RemoveDouble(PString name, int32 index = 0);
	void RemovePointer(PString name, int32 index = 0);

	void RemoveName(PString name);

	// Setting items
	void SetBool(PString name, bool newBool, int32 index = 0);
	void SetInt8(PString name, int8 newInt8, int32 index = 0);
	void SetInt16(PString name, int16 newInt16, int32 index = 0);
	void SetInt32(PString name, int32 newInt32, int32 index = 0);
	void SetInt64(PString name, int64 newInt64, int32 index = 0);
	void SetFloat(PString name, float newFloat, int32 index = 0);
	void SetDouble(PString name, double newDouble, int32 index = 0);
	void SetPointer(PString name, const void *newPointer, int32 index = 0);

	// Getting items
	bool GetBool(PString name, int32 index = 0);
	int8 GetInt8(PString name, int32 index = 0);
	int16 GetInt16(PString name, int32 index = 0);
	int32 GetInt32(PString name, int32 index = 0);
	int64 GetInt64(PString name, int32 index = 0);
	float GetFloat(PString name, int32 index = 0);
	double GetDouble(PString name, int32 index = 0);
	const void *GetPointer(PString name, int32 index = 0);

	// Getting and setting items
	bool GetAndSetBool(PString name, bool newBool, int32 index = 0);
	int8 GetAndSetInt8(PString name, int8 newInt8, int32 index = 0);
	int16 GetAndSetInt16(PString name, int16 newInt16, int32 index = 0);
	int32 GetAndSetInt32(PString name, int32 newInt32, int32 index = 0);
	int64 GetAndSetInt64(PString name, int64 newInt64, int32 index = 0);
	float GetAndSetFloat(PString name, float newFloat, int32 index = 0);
	double GetAndSetDouble(PString name, double newDouble, int32 index = 0);
	const void *GetAndSetPointer(PString name, const void *newPointer, int32 index = 0);

	// Getting and removing items
	bool GetAndRemoveBool(PString name, int32 index = 0);
	int8 GetAndRemoveInt8(PString name, int32 index = 0);
	int16 GetAndRemoveInt16(PString name, int32 index = 0);
	int32 GetAndRemoveInt32(PString name, int32 index = 0);
	int64 GetAndRemoveInt64(PString name, int32 index = 0);
	float GetAndRemoveFloat(PString name, int32 index = 0);
	double GetAndRemoveDouble(PString name, int32 index = 0);
	const void *GetAndRemovePointer(PString name, int32 index = 0);

	// Counting names
	uint32 CountNames(void) const;

	// Getting names
//	void GetNames(PList<PString> *list);

	// Emptying the list
	virtual void MakeEmpty(void);
	virtual bool IsEmpty(void);

protected:

	// An item is a union of different types
	union PItem
	{
		bool aBool;
		int8 aInt8;
		int16 aInt16;
		int32 aInt32;
		int64 aInt64;
		float aFloat;
		double aDouble;
		void *pointer;
	};

	// Entry containing the type and item
	struct PEntry
	{
		PType type;
		PItem item;
	};

	// Helper functions
	void AddItem(PString name, PType type, PItem item);
	void RemoveItem(PString name, PType type, int32 index);
	void SetItem(PString name, PType type, PItem newItem, int32 index);
	PItem GetItem(PString name, PType type, int32 index);
	PItem GetAndSetItem(PString name, PType type, PItem newItem, int32 index);
	PItem GetAndRemoveItem(PString name, PType type, int32 index);

	// Attributes
	PSkipList<PList<PEntry> *, PString> *entryList;
};

#if __p_os == __p_beos && __POWERPC__
#pragma export off
#endif

#endif
